"""
Module for sending data to https://io.adafruit.com/, checkout
https://io.adafruit.com/api/docs/ for more information on the adafruit service.

Example ~/.config/jocasta_config.ini:

[io_adafruit]
username = username
key = api_key_from_adafruit_site

# names of the feeds configured on io.adafruit.com
feeds = office.office-temperature,office.office-light,office.office-humidity

# measurements reported locally.
measurements = temperature,light,humidity

"""


from Adafruit_IO import Client
import logging


logging.basicConfig(
    level=logging.ERROR,
    format='%(asctime)s.%(msecs)03d %(levelname)s: %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S',
)

logger = logging.getLogger(__name__)


class IOAdafruitConnector(object):
    def __init__(self, key: str, username: str, feeds: str, measurements: str) -> None:
        self.aio = Client(username=username, key=key)
        self.feeds = feeds
        self.measurement_mapping = dict(zip(feeds.split(','), measurements.split(',')))

    def send(self, data):
        """
        Update Adafruit feeds.
        """
        for adafruit_feed, measurement_name in self.measurement_mapping.items():
            logger.info(f'Sending {measurement_name} to AdaFruit.')
            self.aio.send_data(adafruit_feed, data[measurement_name])



from typing import Dict
import logging

logging.basicConfig(
    level=logging.ERROR,
    format='%(asctime)s.%(msecs)03d %(levelname)s: %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S',
)

logger = logging.getLogger(__name__)


def validate_temperature(reading: Dict, valid_range: Dict) -> Dict:
    """
    Remove any fields that fall outside like values.
    The ds18b20 returns -127.0 when not connected.

    If a reading has `temperature` in the name, it will be checked, if it falls outside
    the correct range delete it.

>>> validate_temperature({'temperature': '-127.0', 'light': '100'}, \
{'maximum': '50', 'minimum': '-5'})
{'light': 100}
>>> validate_temperature({'temperature': '25.0', 'light': '100'}, \
{'maximum': '50', 'minimum': '-5'})
{'temperature': 25.0, 'light': 100}
>>> validate_temperature({'temperature': '75.0', 'light': '100'}, \
{'maximum': '50', 'minimum': '-5'})
{'light': 100}
    """
    cleaned_data = {}
    for name, value in reading.items():

        try:
            value = float(value)
            max_value = float(valid_range['maximum'])
            min_value = float(valid_range['minimum'])

            if 'temperature' in name:
                if value == -127.0:
                    logger.error(
                        'Invalid reading from ds18b20 sensor of -127.0 '
                        'Please ensure the sensor is connected correctly.'
                    )
                    continue
                elif not value <= max_value or value < min_value:
                    logger.error(
                        f'Reading {value} is outside the configured range '
                        f'{min_value} -> {max_value}'
                    )
                    continue
        except ValueError:
            # bypass any plain text fields
            pass
        cleaned_data[name] = value

    return cleaned_data


if __name__ == "__main__":
    import doctest

    doctest.testmod()



import json

from influxdb import InfluxDBClient
import logging
import platform
from typing import Dict, List

logging.basicConfig(
    level=logging.ERROR,
    format='%(asctime)s.%(msecs)03d %(levelname)s: %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S',
)

logger = logging.getLogger(__name__)


class InfluxDBConnector(object):
    def __init__(self, database, password, username, host=None, port=None):

        if not host:
            host = 'localhost'

        if not port:
            port = 8086

        self.influx_client = InfluxDBClient(host, port, username, password, database)

    def send(self, data: Dict, hostname: str = None) -> None:
        """
        Send the data over to the Influx server.
        """
        json_payload = _build_payload(data, hostname=hostname)
        logger.info('Sending payload to InfluxDB server')
        logger.info(json.dumps(json_payload, indent=2))
        self.influx_client.write_points(json_payload)
        logger.info('Payload sent')


def _build_payload(data: Dict, hostname: str = None) -> List:
    """
    Break out each reading into measurements that Influx will understand.
    """
    logger.info('Building payload for Influxdb')
    payload_values = []

    # location isn't a measurement we want to log.
    location = data.pop('location', 'unset location')

    if not hostname:
        hostname = platform.node()

    for name, value in data.items():
        payload = {
            'measurement': name,
            'tags': {'host': hostname, 'location': location},
            'fields': {'value': float(value)},
        }
        payload_values.append(payload)

    return payload_values



from configparser import ConfigParser, SectionProxy
import typing as t
from pathlib import Path
import os

DEFAULT_HOME = str(Path.home())


def setup_config(ini_file: t.Union[str, None] = None) -> ConfigParser:
    if not ini_file:
        ini_file = os.path.join(DEFAULT_HOME, '.config', 'jocasta_config.ini')
    config = ConfigParser()
    config.read(ini_file)
    return config


def convert_config_stanza(config: SectionProxy) -> t.Dict:
    return {k: v for k, v in config.items()}



import csv
import pathlib
import logging


logging.basicConfig(
    level=logging.ERROR,
    format='%(asctime)s.%(msecs)03d %(levelname)s: %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S',
)

logger = logging.getLogger(__name__)


class CSVFileConnector(object):
    def __init__(self, path):

        self.file_path = path

    def send(self, data) -> bool:
        write_header = False

        # check for file
        path = pathlib.Path(self.file_path)

        if not path.exists():
            write_header = True

        with open(self.file_path, 'a') as csv_file:
            fieldnames = data.keys()
            writer = csv.DictWriter(csv_file, fieldnames=fieldnames)

            # if we have a new file, write the header
            if write_header:
                writer.writeheader()
            writer.writerow(data)
        return True



import setuptools

import jocasta

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='jocasta',
    version=jocasta.__version__,
    author='Chris Hannam',
    author_email='ch@chrishannam.co.uk',
    description='Fetch sensor data.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/chrishannam/jocasta',
    packages=setuptools.find_packages(exclude=('tests', 'examples')),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
    ],
    install_requires=[
        'adafruit-io~=2.4.0',
        'click~=7.1.2',
        'dweepy~=0.3.0',
        'influxdb~=5.3.0',
        'pyserial~=3.4',
        'psutil~=5.7.0',
        'redis~=2.10.6',
        'tabulate~=0.8.7',
    ],
    include_package_data=True,
    entry_points={'console_scripts': ['jocasta=jocasta.collector:main']},
)



"""
Generic collector code to run config file
"""
from typing import Dict

from tabulate import tabulate

from jocasta.inputs.serial_connector import SerialSensor

from jocasta.connectors import file_system, influx, io_adafruit, csv_file

# io_adafruit, influx
from jocasta.command_line.setup import setup_config, convert_config_stanza
import click
import logging

from jocasta.validators import validate_temperature

LEVELS = {
    'critical': logging.CRITICAL,
    'error': logging.ERROR,
    'warn': logging.WARNING,
    'warning': logging.WARNING,
    'info': logging.INFO,
    'debug': logging.DEBUG,
}


@click.command()
@click.option('--port', '-p', type=click.Path(exists=True))
@click.option('--config-file', '-c', required=False, type=click.Path(exists=True))
@click.option('--log-level', '-l', default='error')
def main(port, config_file, log_level):

    level = LEVELS.get(log_level)
    logging.basicConfig(
        level=level,
        format='%(asctime)s.%(msecs)03d %(levelname)s: %(message)s',
        datefmt='%Y-%m-%d %H:%M:%S',
    )

    logger = logging.getLogger(__name__)
    loggers = [logging.getLogger(name) for name in logging.root.manager.loggerDict]
    for logger in loggers:
        logger.setLevel(level)

    logger.debug('Starting...')
    sensor_reader = SerialSensor(port=port)

    reading = sensor_reader.read()

    if reading:
        logger.debug(f'Reading: {reading}')
        config = setup_config(ini_file=config_file)

        connectors: Dict = setup_connectors(config=config)

        display_table(reading)
        if 'temperature_ranges' in config:
            reading = validate_temperature(
                reading, convert_config_stanza(config['temperature_ranges'])
            )
        for name, connector in connectors.items():
            connector.send(data=reading)
    else:
        print('Unable to get reading.')


def setup_connectors(config):
    connectors = {}
    for name, section in config.items():
        args = convert_config_stanza(section)
        if name == 'csv_file':
            connectors[name] = csv_file.CSVFileConnector(**args)
        elif name == 'file_system':
            connectors[name] = file_system.FileSystemConnector(**args)
        elif name == 'io_adafruit':
            connectors[name] = io_adafruit.IOAdafruitConnector(**args)
        elif name == 'influxdb':
            connectors[name] = influx.InfluxDBConnector(**args)
    return connectors


def display_table(reading: Dict):
    table_data = [
        [i.capitalize() for i in reading.keys()],
        [i for i in reading.values()],
    ]
    print(tabulate(table_data, tablefmt='fancy_grid'))


if __name__ == '__main__':
    main()



import json
from typing import Dict


class FileSystemConnector(object):
    def __init__(self, file_name=None):

        if not file_name:
            file_name = '/tmp/jocasta.json'

        self.file_name = file_name

    def send(self, data: Dict) -> bool:
        """
        Write data as JSON to file.
        """

        with open(self.file_name, 'w') as f:
            f.write(json.dumps(data))

        return True



import dweepy


class DweetConnector(object):
    def __init__(self, dweet_name=None):

        self.dweet_name = dweet_name

    def send(self, data):
        # send data to dweet

        try:
            response = dweepy.dweet_for(self.dweet_name, data)
        except Exception as exception:
            response = {'error': exception.message}

        return response



"""
Read data from serial port.
By default we assume the data coming over the serial port
is JSON.

This can be changed to expect csv by:

sensor = SerialSensor(port='ttyUSB0', json_data=False)
"""

import glob
import json
import logging
from json import JSONDecodeError

import serial
import os.path
import time

# usual linux ports
PORTS = ['ttyUSB0', 'ttyUSB1', 'ttyAMA0', 'ttyACM0']
SERIAL_PORT_PATH_ROOT = '/dev/'


logging.basicConfig(
    level=logging.ERROR,
    format='%(asctime)s.%(msecs)03d %(levelname)s: %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S',
)

logger = logging.getLogger(__name__)


class SerialSensor:
    def __init__(self, port=None, json_data=True, debug=True):
        if port:
            self.serial_port = port
        else:
            logger.info('Finding port.')
            self.serial_port = _detect_port()
        self.ser = serial.Serial(self.serial_port, 9600, timeout=1)

        self.debug = debug
        self.json_data = json_data

        logger.debug(f'Using {self.serial_port} as serial port')

    def read(self, timeout=10):

        if not self.serial_port:
            logger.error('Unable to find anything on the serial port to read from.')
            return

        stop = time.time() + timeout
        i = 0

        while time.time() < stop:
            i = i + 1
            raw_serial = self.ser.readline()

            logger.debug(f'Attempt {i}: Received: {raw_serial}')

            if self.json_data and raw_serial:
                try:
                    return json.loads(raw_serial)
                except JSONDecodeError:
                    logger.debug(f'Failed to decode to JSON: {raw_serial}')
            else:
                logger.debug('Failed to decode anything')


def _detect_port():

    device_path = None

    for port in PORTS:
        device = SERIAL_PORT_PATH_ROOT + port
        if os.path.exists(device):
            device_path = device

    if not device_path:
        # lets try osx stuff
        for file_name in glob.glob1('/dev', 'tty.usbserial-*'):
            device_path = SERIAL_PORT_PATH_ROOT + file_name
    return device_path
